#
# Analysis.R 
#
# Written in 2018 by J D Biggins j.biggins@sheffield.ac.uk#

# Purpose: record of the analysis for the article 
# Accurately quantifying the shape of  birds' eggs 
# by J D Biggins, J E Thompson and T R Birkhead
# published in Evolution and Ecology
#
#
# To the extent possible under law, 
# the author has dedicated all copyright and related and 
# neighboring rights to this software to the public domain 
# worldwide. This software is distributed without any warranty.
#
# You should have received a copy of the CC0 Public 
# Domain Dedication along with this software. 
# If not, see <http://creativecommons.org/publicdomain/zero/1.0/>.
#
#
#####     Program NOT fully annotated
#
#------------------------------------------------------------------------------
## ----setglobal, echo=FALSE-----------------------------------------------
#opts_chunk$set(eval=TRUE,  echo=FALSE, cache=TRUE, comment=NA, fig.width=6, fig.height=5.5,out.width='0.9\\textwidth',dpi=600, dev=c('pdf','postscript'),dev.args=list(pdf=list(colormodel='rgb'),postscript=list(colormodel='rgb')))
#opts_chunk$set(eval=TRUE,  echo=FALSE, cache=TRUE, comment=NA, fig.width=6, fig.height=5.5,out.width='0.9\\textwidth',dpi=300, dev=c('pdf','postscript'),dev.args=list(pdf=list(colormodel='cmyk'),postscript=list(colormodel='cmyk')))
options(scipen = 99999999)

 datestamp <- as.character(format(Sys.time(), "%a-%d-%b-%Y-%H-%M"))
 fileforgraphicaloutput <- paste("graphicaloutput-",datestamp, ".pdf",sep="")
# cairo_pdf(onefile=TRUE,file= fileforgraphicaloutput)
 pdf(onefile=TRUE,paper='a4',file= fileforgraphicaloutput)

## ----colour1, echo=FALSE-------------------------------------------------
#knit_hooks$set(document = function(x) {sub('\\usepackage[]{color}', '\\usepackage{xcolor}', x, fixed = TRUE)})

## ----multiplot,   echo=FALSE, results='hide'-----------------------------
library(ggplot2)
# Multiple plot function
#
# ggplot objects can be passed in ..., or to plotlist (as a list of ggplot objects)
# - cols:   Number of columns in layout
# - layout: A matrix specifying the layout. If present, 'cols' is ignored.
#
# If the layout is something like matrix(c(1,2,3,3), nrow=2, byrow=TRUE),
# then plot 1 will go in the upper left, 2 will go in the upper right, and
# 3 will go all the way across the bottom.
#
multiplot <- function(..., plotlist=NULL, file, cols=1, layout=NULL) {
  library(grid)

  # Make a list from the ... arguments and plotlist
  plots <- c(list(...), plotlist)

  numPlots = length(plots)

  # If layout is NULL, then use 'cols' to determine layout
  if (is.null(layout)) {
    # Make the panel
    # ncol: Number of columns of plots
    # nrow: Number of rows needed, calculated from # of cols
    layout <- matrix(seq(1, cols * ceiling(numPlots/cols)),
                    ncol = cols, nrow = ceiling(numPlots/cols))
  }

 if (numPlots==1) {
    print(plots[[1]])

  } else {
    # Set up the page
    grid.newpage()
    pushViewport(viewport(layout = grid.layout(nrow(layout), ncol(layout))))

    # Make each plot, in the correct location
    for (i in 1:numPlots) {
      # Get the i,j matrix positions of the regions that contain this subplot
      matchidx <- as.data.frame(which(layout == i, arr.ind = TRUE))

      print(plots[[i]], vp = viewport(layout.pos.row = matchidx$row,
                                      layout.pos.col = matchidx$col))
    }
  }
}

## ----mainprogram,   echo=FALSE, results='hide'---------------------------

# Need as input

# (i) .csv input file with photonames and egg measurements (egg-measures-test.csv)
#  details are described in main programm

# (ii) egg photos (jpg)
# other photo formats could be used

# Produces 

# (i) .csv file of outputs (test-1.csv)
# (ii) illustrative pictures of fit and circles (test-2-circles.pdf)
# (iii) jpg of each egg, to check that reading the photos has gone properly (photonameoutput.jpg)

# needs the package EBImage which you get as follows:
# 
#  source("http://bioconductor.org/biocLite.R")
#   biocLite("EBImage")
# 
#  library("EBImage")
#
#
#
# Now we have various functions, 
# in the order in which they are called 
# from the main program

#memory.limit(8000)

#################################
#  findegg
#
# This function identifies the egg as a black-white image
# the result is eggimageT
#
findegg=function(eggimage){

# gray scale image
eggimageT=EBImage::channel(eggimage,"gray") 

# convert to binary (thresholds image to black-white) 
eggimageT=eggimageT<EBImage::otsu(eggimageT, levels=65536)        

# identifies the separate parts of the image
imagebits=EBImage::bwlabel(eggimageT) 
    
# computes properties of the bits of the image            
shapes=as.data.frame(EBImage::computeFeatures.shape(imagebits)) 

# identifies the bit of the image that is biggest
eggindex=which(shapes$s.area==max(shapes$s.area)) 

# identifies the bits of the image that are *not* the biggest
indec1=seq(1,max(imagebits))
index=which(indec1!=eggindex) 

# removes all but the largest connected part of the image
eggimageT=EBImage::rmObjects(imagebits,index) 

# return the resulting image
return(eggimageT)
}
#####################end of findegg
########################################

########################################
# eggbox
##this function finds the box around the egg image

eggbox=function(eggimageT){

# sum the columns (sum for each y-coord)
co=colSums(eggimageT) 

# sum the rows    (sum for each x-coord)
ro=rowSums(eggimageT) 

#find the first y-coord with part of the egg
ymin=min(which(co!=0)) 

#find the last y-coord with part of the egg
ymax=max(which(co!=0)) 

#find the first x-coord with part of the egg
xmin=min(which(ro!=0)) 

#find the last x-coord with part of the egg
xmax=max(which(ro!=0)) 

#collect together the coordinates of the bounding box
output=list("ymin"=ymin,"ymax"=ymax,"xmin"=xmin,"xmax"=xmax)
return(output)
}
#######end of eggbox
##########################################

#########################
# orientation
#
# check orientation of the egg
# needs eggimage and the bounding box
#
orientation=function(eggimageT,box)
{
# compute the distance from x at widest point to rightmost point
dup=box$xmax-mean(c(which(eggimageT[,box$ymax]!=0),which(eggimageT[,box$ymin]!=0))) 

# compute distance from x at widest point to leftmost point
ddn=mean(c(which(eggimageT[,box$ymax]!=0),which(eggimageT[,box$ymin]!=0)))-box$xmin 

# compare these: dup>ddn if there is more egg to the RIGHT
# so expect er4 to be positive if more egg to the RIGHT
er4=(dup/ddn)-1 

# so set MOREEGGatLEFT to true if er4<0
MOREEGGatLEFT= (er4<0)
return(MOREEGGatLEFT)
}
#######################end of orientation
#############################################

##########################
# minimal
# 
# a function to cut down the image so that the four edges
# touch the extremes of the eggs
# and to print out a picture of the egg based don this, 
# since this is the key data for the analysis
# it needs the eggimage, the eggname, the bounding box and the flag for printeggs
#

minimal=function(eggimageT,eggname,box,printeggs)
{

# specify the x-coords with some foreground (= egg)
# given from bounding box
nx=seq(box$xmin,box$xmax)

# specify the y-coords with some foreground (=egg)
# given from bounding box
ny=seq(box$ymin,box$ymax) 

# restrict the matrix containing the image to 
# just the rows and columns containing foreground (=egg)
eggimageT=eggimageT[nx,ny]  

# now output the resulting egg image to check it is OK, is desired
if (printeggs)
{ 
#first specify the filename
eggfileoutput=paste(as.character(eggname),"output.jpg",sep="")

# print it; quality=100 for best picture
EBImage::writeImage(eggimageT,eggfileoutput,quality=50)
}
# return the cutdown image
return(eggimageT)
}
################### end of minimal
######################################


######################
# maxplace & minplace
#
# a couple of simple functions whcih find the laregest
# and smallest index of a vector that have a non-zero entry
# 
####
# function to find the largest index of a vector with a non-zero
maxplace=function(x) max(which(x!=0)) 

# function to find the smallest index with a non-zero
minplace=function(x) min(which(x!=0)) 
#
#############end of minplace & maxplace
############################################

#############################################
#
# nextone
#
# recursion to get nth orthogonal polynomial 
# from the previous two
#
# (Gengenbaur with lambda=3/2)
#
# X gives the evaluation points
# C.n.minus.one
# C.n.minus.two
# give the two previous polynomials

nextone=function(C.n.minus.one,C.n.minus.two,n,X)
{
(1/n)*(2*X*(n+lambda-1)*C.n.minus.one- (n+2*lambda-2)*C.n.minus.two )
}
###############end of nextone

#######################################
##
## norm
#
# computes the normalizing constant for the polynomials
# (Gengenbaur with lambda=3/2)
#

norm=function(n)
{(3/4)*2^(1-2*lambda)*pi*gamma(n+2*lambda)/((n+lambda)*gamma(lambda)^2*gamma(n+1))}

############end of norm
######################################################

#####################################
##
##firstderiv
#
# given the coefficients that provide a good fit to the egg
# find the first derivative of the resulting curve
# (formula obtaine dby straightforward calculus)
#

firstderiv=function(c0,c1,c2,c3,X)
{
as.vector(
(c1+2*c2*X+3*c3*X^2)*sqrt(1-X^2)
-(c0+c1*X+c2*X^2+c3*X^3)*X/sqrt(1-X^2)
)
}
########## end of firstderiv
###############################################

################################################
## egg
# 
# this computes the values of the egg profile 
# given the Preston coefficients and the evaluation points X

egg=function(c0,c1,c2,c3,X)
{
as.vector(
(c0+c1*X+c2*X^2+c3*X^3)*sqrt(1-X^2)
)
}
###############end of egg
###########################################################
################################################
## Begg
# 
# this computes the values of the egg profile 
# given the Baker coefficients and the evaluation points X

Begg=function(T,lambda,X)
{
as.vector(
T*(1-X)*((1+X)/(1-X))^(1/(1+lambda))
)
}
###############end of egg
###########################################################

FindCircles <- function(egg){
    #
    # for each possible radius find the (signed horizontal) 
    # distance from the egg outline at the right and left end
    # using the defined function FindMaxDiffCircleEggRight 
    # and FindMaxDiffCircleEggLeft.
    #
    # rightdist contains for each radius ri the maximum 
    # (signed horizontal) distance of an 
    # egg with that radius touching the right end of the egg
    # from the egg itself
    #
    # positive distances correspond to the circle being outside the
    # egg. Do this by successive refinment.
    #  
    ri1 <- (1:10)
    rightdist <- mapply(FUN=FindMaxDiffCircleEggRight, 
                        r=ri1/10, MoreArgs=list(egg=egg))
    #  
    # find the largest radius, rR that is entirely inside 
    # the egg and no more than  a distance zero from the right end
    #
    rR1 <- max(ri1[abs(rightdist)==min(abs(rightdist))])
    
    ri2 <- ((rR1*10-9):((rR1+1)*10-1))
    rightdist <- mapply(FUN=FindMaxDiffCircleEggRight, 
                        r=ri2/100, MoreArgs=list(egg=egg))
    rR2 <- max(ri2[abs(rightdist)==min(abs(rightdist))])
    ri3 <- ((rR2*10-9):((rR2+1)*10)-1)
    rightdist <- mapply(FUN=FindMaxDiffCircleEggRight, 
                        r=ri3/1000, MoreArgs=list(egg=egg))
    rR3 <- max(ri3[abs(rightdist)==min(abs(rightdist))])
    ri4 <- ((rR3*10-9):((rR3+1)*10-1))
    rightdist <- mapply(FUN=FindMaxDiffCircleEggRight, 
                        r=ri4/10000, MoreArgs=list(egg=egg))
    rR4 <- max(ri4[abs(rightdist)==min(abs(rightdist))])
    ri5 <- ((rR4*10-9):((rR4+1)*10)-1)
    rightdist <- mapply(FUN=FindMaxDiffCircleEggRight, 
                        r=ri5/100000, MoreArgs=list(egg=egg))
    rR5 <- max(ri5[abs(rightdist)==min(abs(rightdist))])
    ri6 <- ((rR5*10-9):((rR5+1)*10)-1)
    rightdist <- mapply(FUN=FindMaxDiffCircleEggRight, 
                        r=ri6/1000000, MoreArgs=list(egg=egg))
    rR6 <- max(ri6[abs(rightdist)==min(abs(rightdist))])
    ri7 <- ((rR6*10-9):((rR6+1)*10)-1)
    rightdist <- mapply(FUN=FindMaxDiffCircleEggRight, 
                        r=ri7/1000000, MoreArgs=list(egg=egg))
    rR7 <- max(ri7[abs(rightdist)==min(abs(rightdist))])
    ri8 <- ((rR7*10-9):((rR7+1)*10)-1)
    rightdist <- mapply(FUN=FindMaxDiffCircleEggRight, 
                        r=ri8/10000000, MoreArgs=list(egg=egg))
    rR8 <- max(ri8[abs(rightdist)==min(abs(rightdist))])
    
    
    # do the same for the left end
    ril1 <- (1:10)
    leftdist <- mapply(FUN=FindMaxDiffCircleEggLeft, 
                       r=ril1/10 , MoreArgs=list(egg=egg))
    rL1 <- max(ril1[abs(leftdist)==min(abs(leftdist))])
    ril2 <- ((rL1*10-9):((rL1+1)*10-1))
    leftdist <- mapply(FUN=FindMaxDiffCircleEggLeft, 
                       r=ril2/100 , MoreArgs=list(egg=egg))
    rL2 <- max(ril2[abs(leftdist)==min(abs(leftdist))])
    ril3 <- ((rL2*10-9):((rL2+1)*10-1))
    leftdist <- mapply(FUN=FindMaxDiffCircleEggLeft, 
                       r=ril3/1000 , MoreArgs=list(egg=egg))
    rL3 <- max(ril3[abs(leftdist)==min(abs(leftdist))])
    ril4 <- ((rL3*10-9):((rL3+1)*10-1))
    leftdist <- mapply(FUN=FindMaxDiffCircleEggLeft, 
                       r=ril4/10000 , MoreArgs=list(egg=egg))
    rL4 <- max(ril4[abs(leftdist)==min(abs(leftdist))])
    ril5 <- ((rL4*10-9):((rL4+1)*10-1))
    leftdist <- mapply(FUN=FindMaxDiffCircleEggLeft, 
                       r=ril5/100000 , MoreArgs=list(egg=egg))
    rL5 <- max(ril5[abs(leftdist)==min(abs(leftdist))])
    ril6 <- ((rL5*10-9):((rL5+1)*10-1))
    leftdist <- mapply(FUN=FindMaxDiffCircleEggLeft, 
                       r=ril6/1000000 , MoreArgs=list(egg=egg))
    rL6 <- max(ril6[abs(leftdist)==min(abs(leftdist))])
    ril7 <- ((rL6*10-9):((rL6+1)*10-1))
    leftdist <- mapply(FUN=FindMaxDiffCircleEggLeft, 
                       r=ril7/1000000 , MoreArgs=list(egg=egg))
    rL7 <- max(ril7[abs(leftdist)==min(abs(leftdist))])
    ril8 <- ((rL7*10-9):((rL7+1)*10-1))
    leftdist <- mapply(FUN=FindMaxDiffCircleEggLeft, 
                       r=ril8/10000000 , MoreArgs=list(egg=egg))
    rL8 <- max(ril8[abs(leftdist)==min(abs(leftdist))])
    
    
    rR=rR8/100000000
    rL=rL8/100000000
    
    
    # get the ratio of these two radii
    # which gives Polar Asymmetry (PA)
    PA <- rL/rR 
    # this will be >1, unless more pointed end is too the left
    # return PA and the two radii
    circnew <- data.frame(PA, rR, rL)
    return(circnew)
  }
  # end of FindCircles
  ####################################################
###############################################

#-------------------------------------------------------------------------------
# FindMaxDiffCircleEggRight
# FindMaxDiffCircleEggLeft
#
# (i) for egg-profile egg, computed at XX,
# (ii) for a circle of radius r with centre on the x-axis 
# and touching the egg-profile at its right-hand point 
# (so XX is near 1 hence the [XX>0])
# find the (signed horizontal) distance 
# from the egg-profile, for all XX values near to 1
# and then take the maximum of these.
# Negative distances correspond to the
# point on the circle being inside the egg
# positive ones to being outside
# thus when the maximum is greater than zero the 
# circle goes outside the egg.
# '0' is included explicitly, so a circle wholey inside
# the egg gives a zero 
# 
  FindMaxDiffCircleEggRight <- function(r, egg){
  max(r*cos(asin(egg[(egg/r<1)&(XX>0)]/r))-XX[(egg/r<1)&(XX>0)]-r+1,0)
}
#
# similarly for the end to the left [XX<0]
  FindMaxDiffCircleEggLeft <- function(r, egg){
  max(r*cos(asin(egg[(egg/r<1)&(XX<0)]/r))+XX[(egg/r<1)&(XX<0)]-r+1,0)
}
# end of  FindMaxDiffCircleEggRight &  FindMaxDiffCircleEggLeft
#------------------------------------------------------------------------------

########################################


################################################################
###############
############### main program
###############
#################################################################


# input data in csv format has  at least three columns, and three are named 
# eggname
# maxbreadth
# maxlength
# These are 
# the name of the egg, which is the prefix of the image filename, 
# and the maximum length of the egg 
# and its maximum breadth. 
# These measurements are used in a consistency check on 
# the attributes extracted from the egg image; 
# they are not needed to obtain shape indices, 
# since these are independent of actual lengths. 
# The length is also needed if actual measurements are to be derived 
# (such as the volume).
#eggdata<-data.frame(read.csv("egg-measures-test.csv")) 
eggdata<-data.frame(read.csv("C126.csv")) 

# make rownames the egg identifier
rownames(eggdata)=eggdata$eggname 

# set various varables 
#
# get length of output data
numberofeggs=nrow(eggdata)

# suffix on image filenames 
suffix=".JPG" 

# output a jpg of each egg outline to check against original
#printeggs=TRUE 
printeggs=FALSE

# output graphs showing the biggest  
# circles inside each end of the egg 
# as a check on this aspect working smoothly
printcircles=TRUE 
#printcircles=FALSE

# set lambda to the appropriate value for the Gegenbaur polynomials needed
lambda=3/2 

# for use in drawing circles.
theta=1:1000
theta=2*pi*theta/1000 

# in obtaining Polar Asymmetry circles sought in each end of the egg
# toldiff is the distance the circle can be from the end of the egg; 
# here it is set to zero, so circle must touch the end of the egg
toldiff=0

#########
# Now compute for each egg (here j=1, just one egg) and store
#

j=1


# make character string for file name
eggfilejpg=paste(as.character(eggdata$eggname[j]),suffix,sep="") 

# read the photo of the egg
eggimage <- EBImage::readImage(eggfilejpg)

# define local variables for easier reading
length=eggdata$maxlength[j]
breadth=eggdata$maxbreadth[j]

#
# find the egg in the image 
# and return a black and white (i.e. 0/1) image
# a matrix is returned in which '1' are egg and '0' are not egg
# using the defined function findegg
#
eggimage1=findegg(eggimage)

# eggimage1[x,y] is pixel at (x,y)
  
# matrix rows in eggimage, eggimage[,typical],  
# correspond to horizontal rows (x-coords) of pixels (at y coord=typical)

# matrix columns in eggimage, eggimage[typical,], 
# correspond  vertical columns (y-coords) of pixels (at x coord=typical)


#
# This EBImage function computes various features including the object angle
# in radians: $m.theta
eggprops=as.data.frame(EBImage::computeFeatures.moment(eggimage1))


# rotate image so long axis is horizontal
# eggimage1 is rotated to give eggimage
eggimage=EBImage::rotate(eggimage1,-(360/(2*pi))*eggprops$m.theta)

# for the new image, aligned with its longest axis horizontal,
# identify the smallest box containing the image 
# and create a new image using only this box
# using the defined function eggbox
#
box=eggbox(eggimage)

#check orientation (MORE EGG to the LEFT gives TRUE)
# using the defined function orientation
MOREEGGtoLEFT=orientation(eggimage,box)

# flip egg over if pointed end is at the RIGHT 
# and recompute a couple of things
# (expected not to happen!!)

if (MOREEGGtoLEFT)
{ 
eggimage1=EBImage::rotate(eggimage1,180)
eggprops=as.data.frame(EBImage::computeFeatures.moment(eggimage1))
eggimage=EBImage::rotate(eggimage1,-(360/(2*pi))*eggprops$m.theta)
box=eggbox(eggimage)
}


# replace eggimage by smallest matrix 
# and make output picture of it for a check if
# printeggs=TRUE
# using defined function minimal
#
# eggimage is a matrix with '1' where the egg is and '0' where it isn't
eggimage=minimal(eggimage,eggdata$eggname[j],box,printeggs)

# now get properties of minimal image
# bounding box and Features
box=eggbox(eggimage)
finaleggprops=as.data.frame(EBImage::computeFeatures.moment(eggimage))

# compare the proportions of the image with the measurements
# lengthcheck should be near zero if the photo has the 
# same proportions as the actual measurements
# later those with |lengthcheck|>0.3 will be flagged specifically
lengthcheck=1-(length/breadth)/((box$xmax-box$xmin+1)/(box$ymax-box$ymin+1))

#
# Now for each x coordinate in the image, so for each column of eggimage
# find the largest index where the row is non zero to get maxy
# and the smallest index where the row is non zero to get ymin
# for each x these are the y values for the top and botton of the egg at that x
#
# it uses the defined functions maxplace and minplace
maxy=apply(eggimage,1,maxplace)
miny=apply(eggimage,1,minplace)

#
# create the x variable (on the pixel scale)
# and obtain its overall length
nnx=1:length(maxy)
NNX=max(nnx)

#
# Map the pixel scale x to (-1,1), by forming X.
# X is as in Todd and Smart.
# To avoid zero divisors later, it is actually mapped
# to (-1+epsilon,1-epsilon) with epsilon tiny. 
# check: 
# nnx=1 goes to -(1-epsilon)
# nnx=NNX goes to (1-epsilon)
# linear 
epsilon=0.00000001
X=(1-epsilon)*(-1+(2*(nnx-1)/(NNX-1)))

# NNX pixels correspond to a length of 2 (from -1 to 1)
# so to convert maxy and miny to the same length scale 
# they need to be divided by (NNX/2).
#
# in addition, to combine maxy and miny the midpoint line
# of the egg must be used, this is, on pixel scale
# y=finaleggprops$m.cy
#
# thus on the new scale and centred the variables become
# 2*(maxy-finaleggprops$m.cy)/NNX
# and
# 2*(miny-finaleggprops$m.cy)/NNX
# to reflect the second of these about the x-axis, 
# so that it 'aligns' with the first one, use instead
# 2*(finaleggprops$m.cy-miny)/NNX
#
# Finally to apply the weighted least squares linear modelling 
# we need take these new 'y' values and divide them by sqrt(1-X^2)
# as in the theoretical description

Ymax=2*(maxy-finaleggprops$m.cy)/(NNX*sqrt(1-X^2))
Ymin=2*(finaleggprops$m.cy-miny)/(NNX*sqrt(1-X^2))


#
# Now combine these two Y's, 
# one from the top and one from the bottom of the egg 
# into a single response vector
Y=c(Ymax,Ymin)

#
# the X vector needs to be doubled up correspondingly
X2=c(X,X)



#
# Gegenbaur polynomials (with lambda=3/2) 
# are the relevant ones for weight (1-X^2)
# Use the recursion to find the polynomials
# the defined fuction nextone takes the two previous 
# polynomials evaluated at X2 and computes the next one
# in the sequence; here computed to the fifth power

C0=0*X2+1
C1=2*lambda*X2
C2=nextone(C1,C0,2,X2)
C3=nextone(C2,C1,3,X2)
C4=nextone(C3,C2,4,X2)
C5=nextone(C4,C3,5,X2)
#C6=nextone(C5,C4,6,X2)
#C7=nextone(C6,C5,7,X2)

# These polynomials now need to be normalised
# the defined function norm(i) calculates
# N_i as in the theoretical account

G0w=C0/sqrt(norm(0))
G1w=C1/sqrt(norm(1))
G2w=C2/sqrt(norm(2))
G3w=C3/sqrt(norm(3))
G4w=C4/sqrt(norm(4))
G5w=C5/sqrt(norm(5))
#G6w=C6/sqrt(norm(6))
#G7w=C7/sqrt(norm(7))


#lmG.7=lm(Y~G1w+G2w+G3w+G4w+G5w+G6w+G7w,weights=(1-X2^2))

# fit fifth order polynomial 
# with weights 1-X^2, as described in the theoretical
# presentation
lmG.5=lm(Y~G1w+G2w+G3w+G4w+G5w,weights=(1-X2^2))

# get egg volume from formulae for the fitted egg
# on the scale where egg length is 2
voltemp=(4*pi/3)*sum(coef(lmG.5)^2)

# convert this to cc using measured length in mm of the egg
volcc=voltemp*((length/2)/10)^3

# convert coefficients from orthogonal polynomials to ordinary coefficients
# (i.e. Preston/Todd&Smart), as in theoretical treatment
c3=as.vector((35/2)*sqrt(3/10)*coef(lmG.5)['G3w'])#cubic
c2=as.vector((15/2)*sqrt(7/18)*coef(lmG.5)['G2w'])#quadratic
c1=as.vector(3*sqrt(5/9)*coef(lmG.5)['G1w']+(-15/2)*sqrt(3/10)*coef(lmG.5)['G3w'])#linear
c0=as.vector(coef(lmG.5)['(Intercept)']+(-3/2)*sqrt(7/18)*coef(lmG.5)['G2w'])#constant

# not used here
#firstderivative=(c1+2*c2*X+3*c3*X^2)*sqrt(1-X^2)-(c0+c1*X+c2*X^2+c3*X^3)*X/sqrt(1-X^2)
#surfarea=2*(length/20)^2*mean(2*pi*sqrt(1-X^2)*((c0+c1*X+c2*X^2+c3*X^3))*sqrt(1+(firstderivative)^2))


#find the place where the egg profile has its turning value 
#(highest point on profile)
# by finding the point where the first derivative is zero
# the x-coord of this point is 'fat.point'
fat.point=uniroot(firstderiv,c0=c0,c1=c1,c2=c2,c3=c3,lower=-0.5,upper=0.5)$root

# now use the fit to find 
# the height (the y-coord) at this highest point
# multiply this by 2 to get the maximum breadth 
# of the fitted egg
br=2*(c0+c1*fat.point+c2*fat.point^2+c3*fat.point^3)*sqrt(1-fat.point^2)

# find also the length along the x-axis to the two ends,
# which are at -1 and at 1,
# from fat.point
l2=1-fat.point
l1=1+fat.point

# now compute proportion of the length 
# that is from the widest point to the left end (which is expected to be the more pointy end) end
newpointyratio=l2/2

# and the ratio of the length (=2 in the standardization here) 
# to the maximum breadth 
# as obtained from the fitted egg
neweccen=2/br
#
#
# now start looking for the largest circle in each end of the fitted egg
# first set up a suitable grid of points at which to calculate the 
# egg shape, accuracy is more important near the ends
# so the step length between points is smaller there
# step1X is the basic step length
step1X=0.00001
XX1=seq(from=-0.99999,to=-0.9999,by=step1X)
XX1a=seq(from=-0.9999,to=-0.999,by=10*step1X)
XX2=seq(from=-0.999,to=+0.999,by=100*step1X)
XX3a=seq(from=0.999,to=0.9999,by=10*step1X)
XX3=seq(from=0.9999,to=0.99999,by=step1X)

#put all these together into a single vector of x values 
XX=c(XX1,XX1a,XX2,XX3a,XX3)

# now store the fitted egg, evaluated at XX
thisegg=egg(c0,c1,c2,c3,XX)

#fatpoint.alter=XX[thisegg==max(thisegg)]
# the approach is just a crude search; 
# simply try different radii
# set up
rt1=(1:1000)/100000
rt2=(101:10000)/10000
r.increments=c(rt1,rt2)
#r.increments=(1:10000)/10000
# get the circles, this give the two radii and their ratio
PAplus=FindCircles(thisegg)


#
# If PA<1 then the smaller circle, 
# and hence the 'point' of the egg
# is at the left
#
POINTtoLEFT=(PAplus$PA<1)

# If the more pointed end is at the left
# change PA to be the ratio of the greater radii 
# to the smaller one
# and make newpointyratio the proportion of the length
# from the widest breadth to the more pointed end
# (so it will be <0.5 in these cases)
if(POINTtoLEFT)
{
PAplus$PA=1/PAplus$PA
newpointyratio=1-newpointyratio
}

# fit Baker
y=c(2*(maxy-finaleggprops$m.cy)/NNX,2*(finaleggprops$m.cy-miny)/NNX)

T.start=c0
lambda.start=((coef(lmG.5)['(Intercept)']-coef(lmG.5)['G1w'])/(coef(lmG.5)['(Intercept)']+coef(lmG.5)['G1w']))
Baker=nls(y~Tb*(1-X2)*((1+X2)/(1-X2))^(1/(1+lambda)), start=list(Tb=T.start,lambda=lambda.start))
Tb=coef(Baker)['Tb']
lambda=coef(Baker)['lambda']
fitBaker= mean(residuals(Baker)^2)/4

# fit Baker with more parameters
Baker3.1=nls(y~T3.1*(1-X2)*(1+a3.1*X2+b3.1*X2^2)*((1+X2)/(1-X2))^(1/(1+lambda3.1)), start=list(T3.1=Tb,lambda3.1=lambda,a3.1=0,b3.1=0))
fitBaker3.1= mean(residuals(Baker3.1)^2)/4

# fit Preston by same method (fit is the same)
Preston=nls(y~TTT*(1+C1*X2+C2*X2^2+C3*X2^3)*sqrt(1-X2^2), start=list(TTT=c0+0.001,C1=c1/c0,C2=c2/c0,C3=c3/c0))
currentegg=egg(c0,c1,c2,c3,X2)
fitPreston=mean((currentegg-y)^2)/4
fitPreston.nls= mean(residuals(Preston)^2)/4
# fitPreston1 is the fit for the 6-parameter version, and so is better
# the other two (fitPreston and fitPreston.nls) are the same
fitPreston1=mean((fitted(lmG.5)*sqrt(1-X2^2)-y)^2)/4

# fit Troscianko
alpha.start=-3*sqrt(5/9)*coef(lmG.5)['G1w']/coef(lmG.5)['(Intercept)']

Troscianko=nls(y~TT*(exp(-alpha*X2-beta*X2^2))*sqrt(1-X2^2), start=list(TT=T.start,alpha=alpha.start,beta=0))
TT=coef(Troscianko)['TT']
alpha=coef(Troscianko)['alpha']
beta=coef(Troscianko)['beta']
fitTroscianko= mean(residuals(Troscianko)^2)/4
# fit Troscianko with extra parameter
Troscianko3=nls(y~TT3*(exp(-alpha3*X2-beta3*X2^2-gamma3*X2^3))*sqrt(1-X2^2), start=list(TT3=TT,alpha3=alpha,beta3=beta,gamma3=0))
fitTroscianko3= mean(residuals(Troscianko3)^2)/4

# fit Carter
Carter=nls(y~Tc*(X2+1)^(pC/2)*(1-((X2+1)/2)^pC)^(1/2)/(1-(1/2)^pC)^(1/2),start=list(Tc=c0,pC=-log(newpointyratio)/log(2))) 
Tc=coef(Carter)['Tc']
pC=coef(Carter)['pC']

Carter3=nls(y~Tc3*((X2+1)^(pC3/2)*(1-((X2+1)/2)^pC3)^(1/2)/(1-(1/2)^pC3)^(1/2))*(1+aC*X2+bC*X2^2),start=list(Tc3=Tc,pC3=pC,aC=0,bC=0)) 
#Carter1=nls(y~2*bc*sqrt(((X2+1)/2)^p)*sqrt(1-((X2+1)/2)^p), start=list(bc=br,p=-log(newpointyratio)/log(2)))
fitCarter= mean(residuals(Carter)^2)/4
fitCarter3= mean(residuals(Carter3)^2)/4

# fit polar Carter
# get widest point to be origin for polar coordinates
#  x.at.ymax <- which(eggimage[, box$ymax]!=0)
#  x.at.ymin <- which(eggimage[, box$ymin]!=0)
#  xwideraw <- mean(c(x.at.ymax,x.at.ymin))
#  xwide <- (1-epsilon)*(-1+(2*(xwideraw-1)/(NNX-1)))
# or instead  
 xwide <- mean(X[which((maxy-miny)==max(maxy-miny))])
  # or xwide=1-l1
  # move origin xwide=1
  Xnew=X2-xwide
  thetapolar=atan2(y,Xnew)
  rpolar= sqrt(y^2+(Xnew)^2)
#  # 
#  t1=cos(2*thetapolar)-1
#  t2=-cos(thetapolar)^3
#  t3=-sin(2*thetapolar)^2
#  t4=sin(2*thetapolar)^3
  
 # Ca=lm(rpolar-1~-1+t1+t2+t3+t4)
 
t0=sin(thetapolar)
t1y=(cos(2*thetapolar)-1)*sin(thetapolar)
t2y=(-(cos(thetapolar))^3)*sin(thetapolar)
t3y=(-(sin(2*thetapolar))^2)*sin(thetapolar)
t4y=((sin(2*thetapolar))^3)*sin(thetapolar)

Cay=lm(y~-1+offset(t0)+t1y+t2y+t3y+t4y)


fitCarterpolarCay= mean(residuals(Cay)^2)/4

#fitCarterpolarCa=  mean((((fitted(Ca)+1)*sin(thetapolar))-(rpolar*sin(thetapolar)))^2)/4

#cf.CarterpolarCa=fitCarterpolarCa/fitPreston
cf.CarterpolarCay=fitCarterpolarCay/fitPreston

cf.fitsB=fitBaker/fitPreston
cf.fitsB3=fitBaker3.1/fitPreston
cf.fitsT=fitTroscianko/fitPreston
cf.fitsT3=fitTroscianko3/fitPreston
cf.fitsC=fitCarter/fitPreston
cf.fitsC3=fitCarter3/fitPreston

Bakeregg=Begg(Tb,lambda,XX)
PAplusBaker=FindCircles(Bakeregg)
fat.pointBaker=XX[Bakeregg==max(Bakeregg)]
brBaker=2*Begg(Tb,lambda,fat.pointBaker)
PABaker=PAplusBaker$PA
l2Baker=1-fat.pointBaker


# now compute proportion of the length 
# that is from the widest point to the left end (which is expected to be the more pointy end) end
newpointyratioBaker=l2Baker/2
#

## ----fit-data------------------------------------------------------------

# data from fit comparison

d1 <- data.frame(read.csv("comparison-of-fits.csv"))
d1$Blambda=pmax(d1$Blambda,1/d1$Blambda)
d1$Blambda.Preston=pmax(d1$Blambda.Preston,1/d1$Blambda.Preston)
d1$error.Preston=sqrt(d1$fitPreston)
d1$error.Baker=sqrt(d1$fitPreston*d1$cf.fitsB)
d1$error.Troscianko=sqrt(d1$fitPreston*d1$cf.fitsT)
d1$error.Carter=sqrt(d1$fitPreston*d1$cf.fitsC)
d1$error.Baker3=sqrt(d1$fitPreston*d1$cf.fitsB3)
d1$error.Troscianko3=sqrt(d1$fitPreston*d1$cf.fitsT3)
d1$error.Carter3=sqrt(d1$fitPreston*d1$cf.fitsC3)
d1$error.CarterpolarCay=sqrt(d1$fitPreston*d1$cf.CarterpolarCay)


longd2 <- data.frame(tidyr::gather(d1,method,cf.fit,
                                   c(cf.fitsB,cf.CarterpolarCay,cf.fitsT,
                                     cf.fitsC,cf.fitsB3,cf.fitsC3,cf.fitsT3)))
longd2$method=factor(longd2$method)
longd2$method=dplyr::recode(longd2$method,'cf.fitsB'='Baker',
                            'cf.CarterpolarCay'='CMJ',
                            'cf.fitsT'='Troscianko',
                            'cf.fitsC'='Carter',
                            'cf.fitsB3'='Baker*',
                            'cf.fitsC3'='Carter*',
                            'cf.fitsT3'='Troscianko*')
longd2$method=factor(longd2$method,levels(longd2$method)[c(4,2,6,5,3,7,1)])
d1fitPreston=d1$fitPreston
d1N=d1$N

longd4 <- data.frame(tidyr::gather(d1,method,error,
                                   c(error.Preston,error.Baker,error.Troscianko,
                                     error.Carter,error.CarterpolarCay)))
longd4$method=factor(longd4$method)

longd4$method=dplyr::recode(longd4$method,'error.Baker'='Baker',
                            'error.CarterpolarCay'='CMJ',
                            'error.Troscianko'='Troscianko',
                            'error.Carter'='Carter',
                            'error.Preston'='Preston')
longd4$method=factor(longd4$method,levels(longd4$method)[c(2,1,3,5,4)])

longd5 <- data.frame(tidyr::gather(d1,method,error,
                                   c(error.Preston,error.Baker3,
                                     error.Troscianko3,
                                     error.Carter3)))
longd5$method=factor(longd5$method)

longd5$method=dplyr::recode(longd5$method,'error.Baker3'='Baker*',
                            'error.Carter3'='Carter*',
                            'error.Troscianko3'='Troscianko*',
                            'error.Preston'='Preston')
longd5$method=factor(longd5$method,levels(longd5$method)[c(2,1,4,3)])


## ----get.exteggs---------------------------------------------------------
exteggs=data.frame(read.csv("range-of-egg-shapes.csv"))
exteggs$identifier=1:10
exteggs=exteggs[order(exteggs$Pointedness),]
exteggs$finalorder=1:10

## ----RvHdata, results='hide'---------------------------------------------
eggs <- read.csv("horizontal-v-resting.csv")
eggs$Pointedness=pmax(eggs$Pointedness,1-eggs$Pointedness)
eggs$Blambda=pmax(eggs$Blambda,1/eggs$Blambda)
eggs$FurthestENDtoLeft=as.integer(eggs$FurthestENDtoLeft)
egg2=eggs$egg.code[which(((abs(eggs$lengthcheck)>0.02)&(eggs$position=='horizontal')))]

eggsOK=eggs[!(eggs$egg.code %in% egg2),]
omittedcols=names(eggsOK) %in% 
  c('eggname','SmallCircletoLeft',
    'SmallCirclefurthestfromwidestpoint','check','Lightbackground')

eggsOK.dropcols=data.frame(eggsOK[!omittedcols])
verylong=data.frame(
  tidyr::gather(eggsOK.dropcols, col, value, M0:volcc))
verylong=data.frame(tidyr::unite(verylong,pc,col, position,   sep='.'))
wide.data=data.frame(tidyr::spread(verylong,pc,value))
#adjust coefficients for eggs that are not the same way round each time

oddegg=wide.data$egg.code[abs(wide.data$FurthestENDtoLeft.horizontal-wide.data$FurthestENDtoLeft.resting)>0.5]
#wide.data$c1.horizontal[wide.data$egg.code %in% oddegg]
#wide.data$c1.resting[wide.data$egg.code %in% oddegg]
#wide.data$c3.horizontal[wide.data$egg.code %in% oddegg]
#wide.data$c3.resting[wide.data$egg.code %in% oddegg]

oddeggs=(abs(wide.data$FurthestENDtoLeft.horizontal-wide.data$FurthestENDtoLeft.resting)>0.5)
oddeggs.horiz=(wide.data$FurthestENDtoLeft.horizontal>0.5)&oddeggs
oddeggs.rest=(wide.data$FurthestENDtoLeft.resting>0.5)&oddeggs
    wide.data$c1.horizontal[oddeggs.horiz]=-wide.data$c1.horizontal[oddeggs.horiz]
    wide.data$c3.horizontal[oddeggs.horiz]=-wide.data$c3.horizontal[oddeggs.horiz]
    wide.data$c1.resting[oddeggs.rest]=-wide.data$c1.resting[oddeggs.rest]
    wide.data$c3.resting[oddeggs.rest]=-wide.data$c3.resting[oddeggs.rest]  

wide.data$Pointedness.error <- 
  ((wide.data$Pointedness.resting/wide.data$Pointedness.horizontal)*100) - 100

wide.data$Elongation.error <- 
  ((wide.data$Elongation.resting/wide.data$Elongation.horizontal)*100) - 100

wide.data$PolarAsymmetry.error <- 
  ((wide.data$PolarAsymmetry.resting/wide.data$PolarAsymmetry.horizontal)*100) - 100
wide.data$volcc.error <- 
  ((wide.data$volcc.resting/wide.data$volcc.horizontal)*100) - 100

# From Jamie
#
# level.dataset <- subset(eggsOK, position =="horizontal")
# resting.dataset <- subset(eggsOK, position =="resting")

# Pointedness.error <- ((resting.dataset$Pointedness/level.dataset$Pointedness)*100) - 100
# Elongation.error <- ((resting.dataset$Elongation/level.dataset$Elongation)*100) - 100
# PolarAsymmetry.error <- ((resting.dataset$PolarAsymmetry/level.dataset$PolarAsymmetry)*100) - 100
#
# sum((Pointedness.error-wide.data$Pointedness.error)^2)
# sum((Elongation.error-wide.data$Elongation.error)^2)
# sum((PolarAsymmetry.error-wide.data$PolarAsymmetry.error)^2)
#


wide.data$c0.error <- 
  (wide.data$c0.resting-wide.data$c0.horizontal)

wide.data$c1.error <- 
  (wide.data$c1.resting-wide.data$c1.horizontal)

wide.data$c2.error <- 
  (wide.data$c2.resting-wide.data$c2.horizontal)
wide.data$c3.error <- 
  (wide.data$c3.resting-wide.data$c3.horizontal)


wide.data$Pellipticity.horizontal=(1/wide.data$c0.horizontal)-1

wide.data$B.ellipticity.resting=(1/wide.data$BakerT.resting)-1
wide.data$B.ellipticity.horizontal=(1/wide.data$BakerT.horizontal)-1
wide.data$B.ellipticity.error <- 
  (wide.data$B.ellipticity.resting-wide.data$B.ellipticity.horizontal)
wide.data$B.asymmetry.resting=wide.data$Blambda.resting-1
  wide.data$B.asymmetry.horizontal=wide.data$Blambda.horizontal-1
  wide.data$B.asymmetry.error <- 
    (wide.data$B.asymmetry.resting-wide.data$B.asymmetry.horizontal)

# c1=cor(wide.data$Pellipticity.horizontal,
#         wide.data$B.ellipticity.horizontal, method='spearman')
#  c2=cor(wide.data$Pointedness.horizontal,
#         wide.data$B.asymmetry.horizontal,method='spearman')



## ----getindices----------------------------------------------------------

dataall<-data.frame(read.csv("data-for-comparing-indices.csv"))
dataall$Pointedness=pmax(dataall$Pointedness,1-dataall$Pointedness)
dataall$Blambda.Preston=pmax(dataall$Blambda,1/dataall$Blambda.Preston)

datasome=droplevels(subset(dataall,species=='uria.aalge'
                           |species=='uria.lomvia'
                           |species=='alca.torda'
                           ))


 bk1=egg(dataall$c0,dataall$c1,dataall$c2,dataall$c3,1-dataall$Pointedness) 
  bi1=egg(dataall$c0,dataall$c1,dataall$c2,dataall$c3,-dataall$Pointedness) 
  bi=pmax(bk1,bi1)
  bk=pmin(bk1,bi1)
  
  
  RB= dataall$RadiusLargerCircle/2
  RP= RB/dataall$PolarAsymmetry
  L= 1
  D= 1/ dataall$Elongation
  d= dataall$c0
  
  Preston.asymmetry= (sqrt(RB)-sqrt(RP))*sqrt(L/2)/d
  Preston.bicone= ((sqrt(RB)+sqrt(RP))*sqrt(L/2)/d)-1
  Preston.Asymmetry= (RB-RP)*L/D^2
  Preston.Bicone= ((RB+RP)*L/D^2)-1
  Preston.Bicone1= -Preston.Bicone
  Preston.bicone1= -Preston.bicone
  Mytiai.Matsyura.asymmetry=(RB-RP)/D
  Mytiai.Matsyura.infundibular=RB/D
  Mytiai.Matsyura.infundibular1=1/Mytiai.Matsyura.infundibular
  Mytiai.Matsyura.cloacal=RP/D
  Mytiai.Matsyura.cloacal1=1/Mytiai.Matsyura.cloacal
  Mytiai.Matsyura.interpolar=(L-(RB+RP))/D
  Mytiai.Matsyura.complementarity=(1+(RB/L))*(1+(RP/L))/(1-(RB/L)-(RP/L))
  Mytiai.Matsyura.complementarity1=1/Mytiai.Matsyura.complementarity
  Mand.pearshapedness=(bi-bk)/bi
  Mand.conidity=(bi-bk)/D
  Mand.bluntendconvexity=(2*bi/D)-1
  Mand.sharpendconvexity=(2*bk/D)-1
  Mand.plumpness=3*dataall$volcc/(4*pi*L*D^2)
  Mand.plumpness1=1/Mand.plumpness
  Mand.bluntendconvexity1=-Mand.bluntendconvexity
  Mand.sharpendconvexity1=-Mand.sharpendconvexity
  Elongation=dataall$Elongation
  Pointedness=dataall$Pointedness
  Stoddard.asymmetry=dataall$Blambda.Preston-1
  Stoddard.ellipticity=(1/(dataall$BakerT.Preston))-1
  PolarAsymmetry= dataall$PolarAsymmetry
  Preston.ellipticity=(1/dataall$c0)-1
  
  indices1=cbind(Elongation, Stoddard.ellipticity, Preston.ellipticity,
              Pointedness, Stoddard.asymmetry,
              Preston.asymmetry,Preston.bicone,
              Preston.Asymmetry, Preston.Bicone,
              Mytiai.Matsyura.asymmetry, 
              Mytiai.Matsyura.infundibular,Mytiai.Matsyura.cloacal,
              Mytiai.Matsyura.interpolar,
              Mytiai.Matsyura.complementarity, PolarAsymmetry, 
              Mand.pearshapedness,
              Mand.conidity,Mand.bluntendconvexity,
              Mand.sharpendconvexity,Mand.plumpness)
            
indices2=cbind(Elongation, Stoddard.ellipticity, Preston.ellipticity,
              Mytiai.Matsyura.interpolar,
              Mytiai.Matsyura.complementarity1,Mand.plumpness,
               Mytiai.Matsyura.infundibular1, 
              Preston.bicone, 
             Preston.Bicone,  Mand.bluntendconvexity,
             Pointedness, Stoddard.asymmetry,
               Mand.conidity,
             Mand.sharpendconvexity1, 
              Mand.pearshapedness, Mytiai.Matsyura.cloacal1,
               Preston.asymmetry, Preston.Asymmetry, PolarAsymmetry, 
              Mytiai.Matsyura.asymmetry)

rel.plot=data.frame(Preston.ellipticity,Stoddard.ellipticity,Stoddard.asymmetry,
                    Pointedness)
names(rel.plot)=c('Preston.ellipticity','Stoddard.ellipticity','Stoddard.asymmetry','Pointedness')

co1=cor(rel.plot$Preston.ellipticity,rel.plot$Stoddard.ellipticity,
        method='spearman')
co2=cor(rel.plot$Pointedness, rel.plot$Stoddard.asymmetry, 
        method='spearman')

col4 <- colorRampPalette(c("#7F0000", "red", "#FF7F00", "yellow", "#7FFF7F",
                           "cyan", "#007FFF", "blue", "#00007F"))

## ----subsetauk-----------------------------------------------------------

dataua=droplevels(subset(dataall,species=='uria.aalge'))
dataul=droplevels(subset(dataall,species=='uria.lomvia'))
dataat=droplevels(subset(dataall,species=='alca.torda'))
indexua=data.frame(cbind(dataua$Elongation,dataua$Pointedness,dataua$PolarAsymmetry))
indexul=data.frame(cbind(dataul$Elongation,dataul$Pointedness,dataul$PolarAsymmetry))
indexat=data.frame(cbind(dataat$Elongation,dataat$Pointedness,dataat$PolarAsymmetry))

indices1ua=data.frame(indices1[dataall$species=='uria.aalge',])
indices1ul=data.frame(indices1[dataall$species=='uria.lomvia',])
indices1at=data.frame(indices1[dataall$species=='alca.torda',])

indices2ua=data.frame(indices2[dataall$species=='uria.aalge',])
indices2ul=data.frame(indices2[dataall$species=='uria.lomvia',])
indices2at=data.frame(indices2[dataall$species=='alca.torda',])

namesindex=c('Elongation','Pointedness','PolarAsymmetry')
names(indexua)=namesindex
names(indexul)=namesindex
names(indexat)=namesindex

corua=cor(indexua,method='s')
corul=cor(indexul,method='s')
corat=cor(indexat,method='s')


## ----setforgraphs--------------------------------------------------------
#myred <- rgb(255, 0, 0, max = 255, alpha = 125, names = "red50")
#mygreen<- rgb(0, 255, 0, max = 255, alpha = 75, names = "green30")
#myblue<- rgb(0, 0, 255, max = 255, alpha = 75, names = "blue30")
#myblueviolet<-rgb(138, 43, 226, max = 255, alpha = 75, names = "blueviolet30")
myred <- 'red'
mygreen<- 'green' 
myblue<- 'blue' 
myblueviolet<- 'blueviolet' 

# slip is to arrange vertical alignment of the 
# original egg shape and the fitted egg shape
# did not use this version
# slip1=finaleggprops$m.cy*length/NNX
# slip2=slip1/length

eggname=eggdata$eggname[j]
rP=PAplus$rR
rB=PAplus$rL
r0=PAplus$PA
lineweight1=3
lineweight2=2
lineweight3=2
#
# used this for slip
thisegg1<- egg(c0,c1,c2,c3,X)
MY=max(maxy)
    av1 <- mean((MY-miny)/NNX - (length*thisegg1/2))
    av2 <- mean((MY-maxy)/NNX - (-length*thisegg1/2))
  slip2 <- (av1+av2)/2

  
#
## End of setting up
#
# Start of graphs  

## ----exshapes,  fig.align = 'left',  fig.pos='ht', fig.height=3, out.width='0.95\\textwidth', fig.cap = "The values of the three shape indices for eggs of varied shapes. All egg images are scaled to have the same length. Key:  1) White-breasted Kingfisher (\\textit{Halcyon smyrnensis}); 2) Ad\\'{e}lie penguin (\\textit{Pygoscelis adeliae}); 3) Dalmatian Pelican (\\textit{Pelecanus crispus}); 4) Greater Flamingo (\\textit{Phoenicopterus roseus}); 5) Southern Brown Kiwi (\\textit{Apteryx australis}); 6) Little Grebe (\\textit{Tachybaptus ruficollis}); 7) Royal Tern (\\textit{Thalasseus maximus}); 8) King Penguin (\\textit{Aptenodytes patagonicus}); 9) Pheasant-tailed Jacana (\\textit{Hydrophasianus chirurgus}); 10) Common Guillemot (\\textit{Uria aalge}).",  results='hide'----

#Key:  7); White-breasted Kingfisher (\\textit{Halcyon smyrnensis}); 4) Dalmatian Pelican (\\textit{Pelecanus crispus}); 2) Greater Flamingo (\\textit{Phoenicopterus roseus}); 3) Southern Brown Kiwi (\\textit{Apteryx australis}); 5) Little Grebe (\\textit{Tachybaptus ruficollis})?; 6) Royal Tern (\\textit{Thalasseus maximus}); 1) King Penguin (\\textit{Aptenodytes patagonicus}); 8) Pheasant-tailed Jacana (\\textit{Hydrophasianus chirurgus}); 9) Guillemot (\\textit{Uria aalge})

#
## Start of Figure 1
#
.pardefault <- par(no.readonly = T)  
par(mai=c(0.1, 0.1, 0.1, 0.1),fin=c(6,2))
plot.new()
plot.window(xlim = c(-4, 16.7), ylim = c(-3.5, 1.3), asp=1,xaxs="i",yaxs="i")
box()
text(x=-4,y=-1.9,
     paste('Elongation:'),cex=0.7,pos=4)
text(x=-4,y=-2.5,
     paste('Pointedness:'),cex=0.7,pos=4)
text(x=-4,y=-3.1, 
     paste('PolarAsymmetry:'),cex=0.7,pos=4)
#axis(1) 
#axis(2)
nexteggposition=0.2
for (j in 1:nrow(exteggs)){
  #j=1
  # make character string for file name
  eggname <- as.character(eggdata$eggname[j])
thisegg.ext=egg(exteggs$c0[j],exteggs$c1[j],exteggs$c2[j],exteggs$c3[j],XX)
nextegg=max(thisegg.ext)
eggpoints=c(thisegg.ext, -rev(thisegg.ext))+nexteggposition
eggX=c(rev(XX),XX)
polygon(eggpoints,eggX, col='black')
 #text(x=nexteggposition,y=1.3,exteggs$identifier[j],cex=0.7)
  text(x=nexteggposition,y=1.3,j,cex=0.7)
text(x=nexteggposition,y=-1.9,
     paste(sprintf("%.3f",round(exteggs$Elongation[j]*1000)/1000)),cex=0.7)
text(x=nexteggposition,y=-2.5,
     paste(sprintf("%.3f",round(exteggs$Pointedness[j]*1000)/1000)),cex=0.7)
text(x=nexteggposition,y=-3.1, 
     paste(sprintf("%.3f",round(exteggs$PolarAsymmetry[j]*1000)/1000)),cex=0.7)
nexteggposition=nexteggposition+2*max(thisegg.ext)+0.3
}
par(.pardefault)
#
## ----end of exshapes
#
## End of Figure 1
#

## ----indexpicture,  fig.align = 'center',  fig.pos='ht', out.width='0.5\\textwidth', fig.height=4.5, fig.cap = "Graphical explanation of the symbols occurring in the  text: $L$ is the length of the egg; $D$ is the largest latitudinal diameter; $P$ is the length from the latitude of maximum diameter to the more distant pole; $d$ is the equatorial diameter; $R_B$ and $R_P$ are the radii of the largest circles within the egg and touching the blunt and pointed pole respectively; and $b_i$ and $b_k$ are the latitudinal diameter half way between the latitude of largest diameter and the blunt and pointed pole -- $b_i$ is the larger of the two.",  results='hide'----
# fig.width=5.5, fig.height=4.8,out.width='5.5in',

#
## Start of Figure 2
#

par(mai=c(0, 0, 0, 0),oma=c(1, 1, 1, 1))
plot((1+XX)/2,slip2+thisegg/2,type='l',
     lwd=lineweight3,cex=0.1,asp=1,ylim=c(0,(1/neweccen)),ylab='',xlab='',yaxt = "n",xaxt="n")
points((1+XX)/2,slip2-thisegg/2,type='l',lwd=lineweight3,cex=0.1)
# over plot the circles, at the right end with radius rP
# here toldiff=0
col1=myblue
col2=mygreen
if (rP<rB)
{
  col1=mygreen
  col2=myblue
}
points(((rP*(1+cos(theta))-2*rP+1)-toldiff)/2+1/2,rP*sin(theta)/2+slip2, pch=20,cex=0.1,col=col1)
phi1=pi/2
arrows(((rP*(1+cos(phi1))-2*rP+1)-toldiff)/2+1/2,rP*sin(phi1)/2+slip2,
1-rP/2,br/4,col='black',lwd=1,length=0.1)
arrows(1-rP/2,br/4,((rP*(1+cos(phi1))-2*rP+1)-toldiff)/2+1/2,rP*sin(phi1)/2+slip2,
col='black',lwd=1,length=0.1)
text(x=0.85,y=0.3,expression(R[P]),col='black',pos=4)

# and the other end
points((rB*(1+cos(theta))-1+toldiff)/2+1/2,rB*sin(theta)/2+slip2, pch=20,cex=0.1,col=col2)
phi2=5*pi/4
arrows((rB*(1+cos(phi2))-1+toldiff)/2+1/2,rB*sin(phi2)/2+slip2,
rB/2,br/4,col='black',lwd=1,length=0.1)
arrows(rB/2,br/4,(rB*(1+cos(phi2))-1+toldiff)/2+1/2,rB*sin(phi2)/2+slip2,
col='black',lwd=1,length=0.1)
text(x=0.07,y=0.2,expression(R[B]),col='black',pos=4)

Leps=0.03
arrows(0,br/2+Leps,1,br/2+Leps,col='black',lwd=2,length=0.1)
arrows(1,br/2+Leps,0,br/2+Leps, col='black',lwd=2,length=0.1)
text(x=0.5,y=br/2+Leps+0.03,paste("L",sep=''),col='black',pos=4)

arrows(0.5,br/2, 1,br/2,col='darkblue',lwd=1,length=0.1)
arrows(1,br/2,0.5,br/2, col='darkblue',lwd=1,length=0.1)
text(x=0.73,y=br/2-0.025,paste("L/2",sep=''),col='darkblue',pos=4)


arrows((1+fat.point)/2,0,(1+fat.point)/2,br/2,col='darkmagenta',lwd=2,length=0.1)
arrows((1+fat.point)/2,br/2,(1+fat.point)/2,0,col='darkmagenta',lwd=2,length=0.1)
text(x=0.33,y=0.35,paste("D",sep=''),col='darkmagenta',pos=2)

arrows(1/2,br/4-c0/2,1/2,br/4+c0/2,col='hotpink',lwd=2,length=0.1)
arrows(1/2,br/4+c0/2,1/2,br/4-c0/2,col='hotpink',lwd=2,length=0.1)
text(x=0.5,y=0.37,paste("d",sep=''),col='hotpink',pos=4)

x1=1
#arrows((1+fat.point)/2,br/4,x1,br/4,col='darkgreen',lwd=2,length=0.1)
#arrows(x1,br/4,(1+fat.point)/2,br/4,col='darkgreen',lwd=2,length=0.1)
#text(x=0.6,y=0.2,paste("P",sep=''),
#     col='darkgreen',pos=3)

Peps=-0.03
arrows((1+fat.point)/2,Peps,x1,Peps,col='darkgreen',lwd=2,length=0.1)
arrows(x1,Peps,(1+fat.point)/2,Peps,col='darkgreen',lwd=2,length=0.1)
text(x=0.65,y=-0.055+Peps,paste("P",sep=''),
     col='darkgreen',pos=3)
arrows((1+fat.point)/2,Peps,0,Peps,col='darkgreen',lwd=2,length=0.1)
arrows(0,Peps,(1+fat.point)/2,Peps,col='darkgreen',lwd=2,length=0.1)
text(x=0.14,y=-0.055+Peps,paste("L-P",sep=''),
     col='darkgreen',pos=3)

xbk=(1+fat.point)/2+(1-(1+fat.point)/2)/2
xbi=(1+fat.point)/4
bk=egg(c0,c1,c2,c3,2*xbk-1)/2
bi=egg(c0,c1,c2,c3,2*xbi-1)/2

ybku=br/4+bk
ybkl=br/4-bk
arrows(xbk,0,x1,0,col='darkgreen',lwd=1,length=0.1)
arrows(x1,0,xbk,0,col='darkgreen',lwd=1,length=0.1)
text(x=0.82,y=0,paste("P/2",sep=''),
     col='darkgreen',pos=3)

arrows(xbk,ybku,xbk,ybkl,col='hotpink',lwd=1,length=0.1)
arrows(xbk,ybkl,xbk,ybku,col='hotpink',lwd=1,length=0.1)
text(x=xbk,y=0.34,expression(b[k]),col='hotpink',pos=4)



ybiu=br/4+bi
ybil=br/4-bi
arrows(xbi,0,0,0,col='darkgreen',lwd=1,length=0.1)
arrows(0,0,xbi,0,col='darkgreen',lwd=1,length=0.1)
text(x=xbi/2,y=0,paste("(L-P)/2",sep=''),
     col='darkgreen',pos=3)

arrows(xbi,ybiu,xbi,ybil,col='hotpink',lwd=1,length=0.1)
arrows(xbi,ybil,xbi,ybiu,col='hotpink',lwd=1,length=0.1)
text(x=xbi,y=0.34,expression(b[i]),col='hotpink',pos=4)
par(.pardefault)
#
## ----end of indexpicture
#
## End of Figure 2
#

## ----eggpicture,   fig.align = 'center',  fig.pos='hbt',  out.width='0.6\\textwidth', fig.height=4.5, fig.cap = paste(sep='',"The actual egg shape is the black outline; the Preston fit is in red. The error, as defined at equation \\eqref{error}, is ", round(sqrt(fitPreston),digits=5),". The length of the egg has been scaled to be one. The two circles are the largest possible that touch the end of the egg and are wholly within the (Preston fit to the) egg. Then the Polar Asymmetry (PA) is the ratio of the diameter of the larger (blue) to the smaller (green) circle. Po is the pointedness. El is the Elongation. T is the equatorial diameter."),   results='hide'----
#
## Start of Figure 3
#

# plot the original egg outline from the data; 
# neweccen is used to used to adjust the ylim
# the widest breadth of the egg is length/neweccen
# X runs from -1 to 1, so to make it correspond mm scale mutiply by length/2
#
nX=1:length(X)
  sampleX=(nX%%1==0)
plot((1+X[sampleX])/2,(MY-miny[sampleX])/NNX,col='black',type='l',
     lwd=lineweight1,asp=1,ylim=c(0,(1/neweccen)*1.05),
     ylab='',xlab='',yaxt = "n")
points((1+X[sampleX])/2,(MY-maxy[sampleX])/NNX,
       col='black', type='l', lwd=lineweight1)
points(c(min((1+X)/2),min((1+X)/2)),c((MY-miny[1])/NNX,(MY-maxy[1])/NNX),
       col='black',type='l',lwd=lineweight1)
  points(c(1,1),c((MY-miny[NNX])/NNX,(MY-maxy[NNX])/NNX),col='black',type='l',lwd=lineweight1)
axis(2, at = c(0, 0.2, 0.4, 0.6))
# over plot the egg outline based on XX
  nXX=1:length(XX)
  sampleXX=(nXX%%1==0)
points((1+XX[sampleXX])/2,slip2+thisegg[sampleXX]/2,pch=20,cex=0.1,col=myred)
points((1+XX[sampleXX])/2,slip2-thisegg[sampleXX]/2,pch=20,cex=0.1,col=myred)

# over plot the circles, at the right end with radius rP
# here toldiff=0
col1=myblue
col2=mygreen
if (rP<rB)
{
  col1=mygreen
  col2=myblue
}
points(((rP*(1+cos(theta))-2*rP+1)-toldiff)/2+1/2,rP*sin(theta)/2+slip2, pch=20,cex=0.1,col=col1)

# and the other end
points((rB*(1+cos(theta))-1+toldiff)/2+1/2,rB*sin(theta)/2+slip2, pch=20,cex=0.1,col=col2)

# label
#legend("topleft",legend=paste(as.character(eggdata$eggname[1])),cex=0.6)


arrows((1+fat.point)/2,0,(1+fat.point)/2,br/2,col='darkmagenta',lwd=2)
arrows((1+fat.point)/2,br/2,(1+fat.point)/2,0,col='darkmagenta',lwd=2)
text(x=0.33,y=0.35,paste("1/El=",signif(br/2,3),sep=''),col='darkmagenta',pos=2)

arrows(1/2,br/4-c0/2,1/2,br/4+c0/2,col='hotpink',lwd=2)
arrows(1/2,br/4+c0/2,1/2,br/4-c0/2,col='hotpink',lwd=2)
text(x=0.51,y=0.37,paste("T=",signif(c0,3),sep=''),col='hotpink',pos=4)

x1=1
if(POINTtoLEFT)
{
x1=0
}
arrows((1+fat.point)/2,br/4,x1,br/4,col='darkgreen',lwd=2)
arrows(x1,br/4,(1+fat.point)/2,br/4,col='darkgreen',lwd=2)
text(x=0.6,y=0.2,paste("Po=",signif((1-fat.point)/2,3),sep=''),
     col='darkgreen',pos=3)
text(x=1,y=0,paste("PA=",signif(rB/rP,3),sep=''),
                   col='blueviolet',pos=2)

#
## ----end of eggpicture
#
# End of Figure 3
#

## ----Bakerpicture,  fig.align = 'center',  fig.pos='hbt',  out.width='0.6\\textwidth', fig.height=4.5, fig.cap = paste(sep='',"The actual egg shape is the black outline; the Baker fit is in red. The error, as defined at equation \\eqref{error}, is ", round(sqrt(fitBaker),digits=5),". The values of Po, 1/El and T based on the Baker fit are indicated. The Polar Asymmetry (PA) is very large because of the very small circle at the more pointed end."),   results='hide'----
#
## Start of Figure 4
#
par(mar=c(2, 2, 2, 2),oma=c(1, 1, 1, 1))
nX=1:length(X)
  sampleX=(nX%%1==0)
plot((1+X[sampleX])/2,(MY-miny[sampleX])/NNX,col='black',type='l',lwd=lineweight2,asp=1,ylim=c(0,(1/neweccen)*1.05),ylab='',xlab='',yaxt = "n")
points((1+X[sampleX])/2,(MY-maxy[sampleX])/NNX,col='black',type='l',lwd=lineweight2)
points(c(min((1+X)/2),min((1+X)/2)),c((MY-miny[1])/NNX,(MY-maxy[1])/NNX),
       col='black',type='l',lwd=lineweight2)
  points(c(1,1),c((MY-miny[NNX])/NNX,(MY-maxy[NNX])/NNX),
         col='black',type='l',lwd=lineweight2)
axis(2, at = c(0, 0.2, 0.4, 0.6))  
# over plot the egg outline based on XX
points((1+XX)/2,slip2+Bakeregg/2,pch=20,cex=0.1,col=myred)
points((1+XX)/2,slip2-Bakeregg/2,pch=20,cex=0.1,col=myred)
#legend("topleft",legend=paste(as.character(eggdata$eggname[1])),cex=0.6)
arrows((1+fat.pointBaker)/2,max(slip2+Bakeregg/2),(1+fat.pointBaker)/2,min(slip2-Bakeregg/2),col='darkmagenta',lwd=2)
arrows((1+fat.pointBaker)/2,min(slip2-Bakeregg/2),(1+fat.pointBaker)/2,max(slip2+Bakeregg/2),col='darkmagenta',lwd=2)
mid=(max(slip2+Bakeregg/2)+min(slip2-Bakeregg/2))/2
text(x=0.35,y=0.35,paste("1/El=",signif(max(Bakeregg),3),sep=''),col='darkmagenta',pos=2)

arrows(1/2,mid-Tb/2,1/2,mid+Tb/2,col='hotpink',lwd=2)
arrows(1/2,mid+Tb/2,1/2,mid-Tb/2,col='hotpink',lwd=2)
text(x=0.51,y=0.37,paste("T=",signif(Tb,3),sep=''),col='hotpink',pos=4)

arrows((1+fat.pointBaker)/2,br/2+0.03,x1,br/2+0.03,col='darkgreen',lwd=2)
arrows(x1,br/2+0.03,(1+fat.pointBaker)/2,br/2+0.03,col='darkgreen',lwd=2)
text(x=0.73,y=br/2+0.02,paste("Po=",signif((1-fat.pointBaker)/2,3),sep=''),
     col='darkgreen',pos=1)

# over plot the circles, at the right end with radius rP
# here toldiff=0
points(((PAplusBaker$rR*(1+cos(theta))-2*PAplusBaker$rR+1)-toldiff)/2+1/2,PAplusBaker$rR*sin(theta)/2+slip2, pch=20,cex=0.1,col=col1)
points((PAplusBaker$rL*(1+cos(theta))-1+toldiff)/2+1/2,PAplusBaker$rL*sin(theta)/2+slip2, pch=20,cex=0.1,col=col2)
text(x=1,y=0,paste("PA=",signif(PAplusBaker$rL/PAplusBaker$rR,3),sep=''),
                   col='blueviolet',pos=2)
par(.pardefault)
#
## ----end of Bakerpicture
#
# End of Figure 4
#

## ----Fboxplot-new,  fig.align = 'center',  out.width='\\textwidth', fig.height=4.5, fig.pos='bht', fig.cap = paste("Boxplots comparing the error defined at \\eqref{error} (multiplied by 1000) for the methods of  \\citet{Ca}, \\citet{Ba} and \\citet{Tr}, given in equations \\eqref{e:Carter}, \\eqref{e:Baker} and \\eqref{e:Troscianko}, the method of \\citet{CJ}, described in equation \\eqref{e:CMJ}, labelled CMJ, and the method \\citet{P1} given in equation \\eqref{e:Preston3}. The results are for", length(d1$N), "eggs of various species: ",length(d1$species[d1$species=='uria.aalge']), "\\textit{Uria aalge}, ", length(d1$species[d1$species=='uria.lomvia']), "\\textit{Uria lomvia},", length(d1$species[d1$species=='alca.torda']), "\\textit{Alca torda}, ", length(d1$species[d1$species=='Aptenodytes patagonicus']),"\\textit{Aptenodytes patagonicus}, ", length(d1$species[d1$species=='Lanius.collurio'])," \\textit{Lanius collurio}, ", length(d1$species[d1$species=='Phalacrocorax.carbo'])," \\textit{Phalacrocorax carbo}, ", length(d1$species[d1$species=='Gallus.gallus.domesticus'])," \\textit{Gallus gallus domesticus}, ", length(d1$species[d1$species=='Spheniscus humboldti'])," \\textit{Spheniscus humboldti}, ", length(d1$species[d1$species=='Eudyptes pachyrhynchus'])," \\textit{Eudyptes pachyrhynchus}, ", length(d1$species[d1$species=='Larus Fuscus'])," \\textit{Larus fuscus}. The heavy line is the median, the boxes extend between the upper and lower quartiles, the whiskers extend to the minimum and maximum.",sep=' '),   results='hide'----

#
## Start of Figure 5
#

#boxplot(sqrt(longd2$cf.fit)~as.factor(longd2$method),xlab='Method',ylab="ratio of Method's fit to Preston's fit",cex.axis=0.75,cex.lab=0.8)
#points(sqrt(longd2$cf.fit)[longd2$eggname=='C126']~as.factor(longd2$method)[longd2$eggname=='C126'],col='red',pch=20)

boxplot(1000*longd4$error~as.factor(longd4$method),range=0,xlab='Method',ylab=expression(error%*%10^3))#,cex.axis=0.75,cex.lab=0.8)
#
## ----end of Fboxplot-new
#
# End of Figure 5
#

## ----Fboxplot-new2,  fig.align = 'center',  fig.pos='htb', out.width='\\textwidth', fig.height=4.5, fig.cap = paste("Boxplots comparing the error defined at \\eqref{error} (multiplied by 1000) for Preston's proposal, given in \\eqref{e:Preston3}, and four-parameter versions of the proposals  of  \\citet{Ca}, \\citet{Ba} and \\citet{Tr}, introduced in the present study, in equations \\eqref{Carter4}, \\eqref{Baker3} and \\eqref{Troscianko3}, respectively, and indicated with a `*'. The results are for the", length(d1$N), "eggs used also in Fig. \\ref{fig:Fboxplot-new}. The heavy line is the median, the boxes extend between the upper and lower quartiles, the whiskers extend to the minimum and maximum.",sep=' '),   results='hide'----
#
#
## Start of Figure SF1
#
boxplot(1000*longd5$error~as.factor(longd5$method),range=0,ylim=c(0,5),xlab='Method',ylab=expression(error%*%10^3))#,cex.axis=0.75,cex.lab=0.8)
#
## ----end of Fboxplot-new2
#
# End of Figure SF1
#

## ----Fboxplot,  fig.align = 'center',  fig.pos='htb', out.width='\\textwidth',  fig.cap = paste("Boxplots comparing the error, defined at \\eqref{error}, from Preston's method using equation \\eqref{e:Preston3}, with: the proposals of  \\citet{Ca}, \\citet{Ba} and \\citet{Tr}, given in equations \\eqref{e:Carter}, \\eqref{e:Baker} and \\eqref{e:Troscianko}; with their four-parameter variants introduced in the present study, in equations \\eqref{Carter4}, \\eqref{Baker3} and \\eqref{Troscianko3}, indicated with a `*'; and with the proposal of \\citet{CJ}, decribed in equation \\eqref{e:CMJ} and labelled CMJ. For each egg, the logarithm of the ratio of the method's error to the Preston error was obtained. The results are for the", length(d1$N), "eggs used also in Fig. \\ref{fig:Fboxplot-new}. The heavy line is the median, the boxes extend between the upper and lower quartiles, the whiskers extend to the minimum and maximum. The points are for the common Guillemot egg C126, used to illustrate the fit in Figs \\ref{fig:eggpicture}, \\ref{fig:Bakerpicture}, \\ref{fig:Carter4picture} and \\ref{fig:Carterpolarpicture}.",sep=' '),   results='hide'----
#
## Start of Figure SF2
#
#boxplot(sqrt(longd2$cf.fit)~as.factor(longd2$method),xlab='Method',ylab="ratio of Method's fit to Preston's fit",cex.axis=0.75,cex.lab=0.8)
#points(sqrt(longd2$cf.fit)[longd2$eggname=='C126']~as.factor(longd2$method)[longd2$eggname=='C126'],col='red',pch=20)
boxplot(log(longd2$cf.fit)/2~as.factor(longd2$method),range=0,xlab='Method',ylab="logarithm of ratio of Method's error to Preston's error",cex.axis=0.75,cex.lab=0.8)
points(log(longd2$cf.fit)[longd2$eggname=='C126']/2~as.factor(longd2$method)[longd2$eggname=='C126'],col='red',pch=20)
#legend("topright", legend= 'Egg C126', col='red',pch=20)
#
## ----end of Fboxplot
#
# End of Figure SF2
#

## ----RvH,  fig.align = 'center',  fig.pos='ht', fig.height=5.5, fig.cap = "For each of the four characteristics (Elongation, Pointedness, Polar Asymmetry, volume), the percentage error from a  photograph in a resting position is plotted against the value of the characteristic for a photograph in a horizontal position.",   results='hide'----

#
## Start of Figure SF3
#

# from Jamie
#
# plot(level.dataset$Pointedness,Pointedness.error, cex.axis = 0.5, col = "black", pch = 16)
#
#plot(level.dataset$Elongation,Elongation.error, cex.axis = 0.5, col = "black", pch = 16)
#
#plot(level.dataset$PolarAsymmetry,PolarAsymmetry.error, cex.axis = 0.5, col = "black", pch = 16)

p1 <- ggplot(wide.data, aes(x=Elongation.horizontal, y=Elongation.error)) +
    geom_point(size=1) +
    xlab('Elongation')+
    ylab("% error")+
    theme_bw() +
  theme(panel.grid=  element_blank(),legend.title=element_blank())#+
      #ggtitle("Elongation")
p2 <- ggplot(wide.data, aes(x=Pointedness.horizontal, y=Pointedness.error)) +
    geom_point(size=1) +
    xlab('Pointedness')+
    ylab("% error")+  
  theme_bw() +
  theme(panel.grid=  element_blank(),legend.title=element_blank())#+
  #ggtitle("Pointedness")
p3 <- ggplot(wide.data, aes(x=PolarAsymmetry.horizontal, y=PolarAsymmetry.error)) +
    geom_point(size=1) +
  xlab('Polar Asymmetry')+
    ylab("% error")  +
  theme_bw() +
  theme(panel.grid=  element_blank(),legend.title=element_blank())#+
    #ggtitle("PolarAsymmetry")
 

   p4 <- ggplot(wide.data, aes(x=volcc.horizontal, y=volcc.error)) +
    geom_point(size=1) +
  xlab((expression(Volume~(cm^3))))+
    ylab("% error")  +
  theme_bw() +
  theme(panel.grid=  element_blank(),legend.title=element_blank())#+
    #ggtitle("PolarAsymmetry")     
multiplot(p1, p2, p3, p4, cols=2)
#
## ----end of RvH
#
# End of Figure SF3
#

## ----RvH2,  fig.align = 'center',  fig.pos='ht', fig.height=5.5 ,fig.cap = "For each of the four Preston parameters, the difference in the value between  the egg photographed in a resting position and in a horizontal position is plotted against the value of the parameter from the photograph in a horizontal position",   results='hide'----

#
## Start of Figure SF4
#

p1c <- ggplot(wide.data, aes(x=c0.horizontal, y=c0.error)) +
    geom_point(size=1) +
    xlab(expression(c[0]))+
    ylab("difference")+
    theme_bw() +
  theme(panel.grid=  element_blank(),legend.title=element_blank())#+
      #ggtitle("Elongation")
 p2c <- ggplot(wide.data, aes(x=c1.horizontal, y=c1.error)) +
    geom_point(size=1) +
    xlab(expression(c[1]))+
    ylab("difference")+  
  theme_bw() +
  theme(panel.grid=  element_blank(),legend.title=element_blank())#+
  #ggtitle("Pointedness")
p3c <- ggplot(wide.data, aes(x=c2.horizontal, y=c2.error)) +
    geom_point(size=1) +
  xlab(expression(c[2]))+
    ylab("difference")  +
  theme_bw() +
  theme(panel.grid=  element_blank(),legend.title=element_blank())#+
    #ggtitle("PolarAsymmetry")

 p4c <- ggplot(wide.data, aes(x=c3.horizontal, y=c3.error)) +
    geom_point(size=1) +
  xlab(expression(c[3]))+
    ylab("difference")  +
  theme_bw() +
  theme(panel.grid=  element_blank(),legend.title=element_blank())#+
    #ggtitle("PolarAsymmetry")   
    
multiplot(p1c, p3c, p2c,  p4c, cols=2)
#
## ----end of RvH2
#
# End of Figure SF4
#

## ----RvH3,  fig.align = 'center',  fig.pos='ht', fig.height=2.75 ,fig.cap = paste("For \\citepos{St} ellipticity and asymmetry, the difference in the value between  the egg photographed in a resting position and in a horizontal position is plotted against the value from the photograph in a horizontal position. ",sep=''),   results='hide'----

#
## Start of Figure SF5
#

p2<- ggplot(wide.data, aes(x=B.ellipticity.horizontal, y=B.ellipticity.error)) +
    geom_point(size=1) +
    xlab('ellipticity (horizontal)')+
    ylab("difference")+
    theme_bw() +
  theme(panel.grid=  element_blank(),legend.title=element_blank())#+

  p3<- ggplot(wide.data, aes(x=B.asymmetry.horizontal, y=B.asymmetry.error)) +
    geom_point(size=1) +
    xlab('asymmetry (horizontal)')+
    ylab("difference")+
    theme_bw() +
    theme(panel.grid=  element_blank(),legend.title=element_blank())#+
  
multiplot( p2,   p3, cols=2)
#
## ----end of RvH3
#
# End of Figure SF5
#

## ----cor-pic1, fig.align = 'left', out.width='\\textwidth', fig.pos='ht',fig.cap = paste("For the photographs of ", length(indexua$Elongation), " \\textit{Uria aalge} eggs, Spearman correlation between various indices described in Table \\ref{t1} and also Preston.ellipticity, given by $(1/c_0)-1$ where $c_0$ is the first Preston parameter. Values are represented by colour and circle size in the upper triangle; the numerical values (times 100) are given in the lower triangle.",sep=''),   results='hide'----
#
## Start of Figure SF6
#

cor1ua=cor(indices1ua,method='spearman')
cor2ua=cor(indices2ua,method='spearman')

 corrplot::corrplot.mixed(cor1ua, lower = "number", upper = "circle", 
                          tl.pos ="lt", diag = "n", number.digits=1,
                          number.cex=0.36,  tl.cex=0.65, tl.col='black',
                          order='original', 
                          upper.col=col4(200),lower.col='black',
                          cl.lim = c(-1,1),cl.length=11,
                          addCoefasPercent=TRUE,cl.cex=0.5,cl.ratio=0.18)
 
#
## ----end of cor-pic1
#
# End of Figure SF6
#

## ----cor-pic2, fig.align = 'left',  fig.pos='ht',out.width='\\textwidth', fig.cap = paste("For the photographs of ", length(indexua$Elongation), " \\textit{Uria aalge} eggs, Spearman correlation between various indices. The indices are as  in Fig.\ \\ref{fig:cor-pic1} with the following exceptions.  Mand.sharpendconvexity1 is $-1$ times the original index, and Mytiai.Matsyura.infundibular1, Mytiai.Matsyura.cloacal1 and Mytiai.Matsyura.complementarity1 are the reciprocals of the original index. Values are represented by colour and circle size in the upper triangle; the numerical values (times 100) are given in the lower triangle.",sep=''),   results='hide'----
#
## Start of Figure SF7
#

corrplot::corrplot.mixed(cor2ua, lower = "number", upper = "circle", 
                          tl.pos ="lt", diag = "n", number.digits=1,
                          number.cex=0.36, tl.cex=0.65, tl.col='black',
                          order='original',  
                          upper.col=col4(20),lower.col='black',
                          cl.lim = c(-0.8,1),cl.length=10,
                          addCoefasPercent=TRUE,cl.cex=0.5,cl.ratio=0.18)
#
## ----end of cor-pic2
#
# End of Figure SF7
#

## ----cor-pic3, fig.align = 'left',  fig.pos='ht',out.width='\\textwidth', fig.cap = paste("For the photographs of ", length(indexul$Elongation), " \\textit{Uria lomvia} eggs, Spearman correlation between various indices. The indices are as  in Fig.\ \\ref{fig:cor-pic1} with the following exceptions.  Mand.sharpendconvexity1 is $-1$ times the original index, and Mytiai.Matsyura.infundibular1, Mytiai.Matsyura.cloacal1 and Mytiai.Matsyura.complementarity1 are the reciprocals of the original index. Values are represented by colour and circle size in the upper triangle; the numerical values (times 100) are given in the lower triangle.",sep=''),   results='hide'----
#
## Start of Figure SF8
#

  cor2ul=cor(indices2ul,method='spearman')
corrplot::corrplot.mixed(cor2ul, lower = "number", upper = "circle", 
                          tl.pos ="lt", diag = "n", number.digits=1,
                          number.cex=0.36, tl.cex=0.65, tl.col='black',
                          order='original',  
                          upper.col=col4(20),lower.col='black',
                           cl.lim = c(-0.8,1),cl.length=10,
                          addCoefasPercent=TRUE,cl.cex=0.5,cl.ratio=0.18)

#
## ----end of cor-pic3
#
# End of Figure SF8
#

## ----cor-pic4, fig.align = 'left',  fig.pos='ht',out.width='\\textwidth', fig.cap = paste("For the photographs of ", length(indexat$Elongation), " \\textit{Alca torda} eggs, Spearman correlation between various indices. The indices are as  in Fig.\ \\ref{fig:cor-pic1}  with the following exceptions.  Mand.sharpendconvexity1 is $-1$ times the original index, and Mytiai.Matsyura.infundibular1, Mytiai.Matsyura.cloacal1 and Mytiai.Matsyura.complementarity1 are the reciprocals of the original index. Values are represented by colour and circle size in the upper triangle; the numerical values (times 100) are given in the lower triangle.",sep=''),   results='hide'----
#
## Start of Figure SF9
#

  cor2at=cor(indices2at,method='spearman')
corrplot::corrplot.mixed(cor2at, lower = "number", upper = "circle", 
                          tl.pos ="lt", diag = "n", number.digits=1,
                          number.cex=0.36, tl.cex=0.65, tl.col='black',
                          order='original',  
                          upper.col=col4(20),lower.col='black',
                           cl.lim = c(-0.8,1),cl.length=10,
                          addCoefasPercent=TRUE,cl.cex=0.5,cl.ratio=0.18)

#
## ----end of cor-pic4
#
# End of Figure SF9
#

## ----Trosciankopicture,  fig.align = 'center',  fig.pos='hbt',  out.width='0.6\\textwidth', fig.height=4.5, fig.cap = paste(sep='',"The actual egg shape is the black outline; the Troscianko fit is in red. The error, as defined at equation \\eqref{error}, is ", round(sqrt(fitTroscianko),digits=5),". The values of Po, 1/El and T based on the Troscianko fit are indicated. At ",signif(PAplusTroscianko$rL/PAplusTroscianko$rR,3), " the Polar Asymmetry here is markedly larger than the value of ", signif(rB/rP,3)," illustrated in Fig. \\ref{fig:eggpicture} because the Troscianko fit is too pointed at the pointed end."),   results='hide'----
#
## Start of Figure SF11
#  
  Tegg=function(TT,alpha,beta,X)
  {
    as.vector(
      TT*(exp(-alpha*X-beta*X^2))*sqrt(1-X^2)
    )
  }
par(mar=c(2, 2, 2, 2),oma=c(1, 1, 1, 1))


  TrosciankoEgg=Tegg(TT,alpha,beta,XX)
  PAplusTroscianko=FindCircles(TrosciankoEgg)
  fat.pointTroscianko=XX[TrosciankoEgg==max(TrosciankoEgg)]
  nX=1:length(X)
  sampleX=(nX%%1==0)
  plot((1+X[sampleX])/2,(MY-miny[sampleX])/NNX,col='black',type='l',lwd=lineweight2,asp=1,ylim=c(0,(1/neweccen)*1.05),ylab='',xlab='',yaxt = "n")
  points((1+X[sampleX])/2,(MY-maxy[sampleX])/NNX,col='black',type='l',lwd=lineweight2)
  points(c(min((1+X)/2),min((1+X)/2)),c((MY-miny[1])/NNX,(MY-maxy[1])/NNX),
         col='black',type='l',lwd=lineweight2)
  points(c(1,1),c((MY-miny[NNX])/NNX,(MY-maxy[NNX])/NNX),
         col='black',type='l',lwd=lineweight2)
  axis(2, at = c(0, 0.2, 0.4, 0.6))  
  # over plot the egg outline based on XX
  points((1+XX)/2,slip2+TrosciankoEgg/2,pch=20,cex=0.1,col=myred)
  points((1+XX)/2,slip2-TrosciankoEgg/2,pch=20,cex=0.1,col=myred)
  
  
  #legend("topleft",legend=paste(as.character(eggdata$eggname[1])),cex=0.6)
  arrows((1+fat.pointTroscianko)/2,max(slip2+TrosciankoEgg/2),(1+fat.pointTroscianko)/2,min(slip2-TrosciankoEgg/2),col='darkmagenta',lwd=2)
  arrows((1+fat.pointTroscianko)/2,min(slip2-TrosciankoEgg/2),(1+fat.pointTroscianko)/2,max(slip2+TrosciankoEgg/2),col='darkmagenta',lwd=2)
  mid=(max(slip2+TrosciankoEgg/2)+min(slip2-TrosciankoEgg/2))/2
  text(x=0.34,y=0.35,paste("1/El=",signif(max(TrosciankoEgg),3),sep=''),col='darkmagenta',pos=2)
  
  arrows(1/2,mid-TT/2,1/2,mid+TT/2,col='hotpink',lwd=2)
  arrows(1/2,mid+TT/2,1/2,mid-TT/2,col='hotpink',lwd=2)
  text(x=0.51,y=0.37,paste("T=",signif(TT,3),sep=''),col='hotpink',pos=4)
  
  arrows((1+fat.pointTroscianko)/2,br/2+0.03,x1,br/2+0.03,col='darkgreen',lwd=2)
  arrows(x1,br/2+0.03,(1+fat.pointTroscianko)/2,br/2+0.03,col='darkgreen',lwd=2)
  text(x=0.73,y=br/2+0.02,paste("Po=",signif((1-fat.pointTroscianko)/2,3),sep=''),
       col='darkgreen',pos=1)
  
  # over plot the circles, at the right end with radius rP
  # here toldiff=0
  points(((PAplusTroscianko$rR*(1+cos(theta))-2*PAplusTroscianko$rR+1)-toldiff)/2+1/2,PAplusTroscianko$rR*sin(theta)/2+slip2, pch=20,cex=0.1,col=col1)
  points((PAplusTroscianko$rL*(1+cos(theta))-1+toldiff)/2+1/2,PAplusTroscianko$rL*sin(theta)/2+slip2, pch=20,cex=0.1,col=col2)
  text(x=1,y=0,paste("PA=",signif(PAplusTroscianko$rL/PAplusTroscianko$rR,3),sep=''),
       col='blueviolet',pos=2)
  par(.pardefault)
  #
## ----end of Trosciankopicture
#
# End of Figure SF11
#

## ----Carter4picture,  fig.align = 'center',  fig.pos='ht', out.width='0.6\\textwidth', fig.height=4.5,  fig.cap = "\\label{F3}  The actual egg shape is the black outline; the 4 parameter Carter-fit, based on equation \\eqref{Carter4}, is in red.",   results='hide'----
#
## Start of Figure SF12
# 
  
Cegg3=function(Tc,pC,aC,bC,X)
{
  as.vector(
    Tc*((X+1)^(pC/2)*(1-((X+1)/2)^pC)^(1/2)/(1-(1/2)^pC)^(1/2))*(1+aC*X+bC*X^2)
  )
}
Tc=coef(Carter3)['Tc3']
pC=coef(Carter3)['pC3']
aC=coef(Carter3)['aC']
bC=coef(Carter3)['bC']
Carteregg3=Cegg3(Tc,pC,aC,bC,XX)

nX=1:length(X)
  sampleX=(nX%%1==0)
plot((1+X[sampleX])/2,(MY-miny[sampleX])/NNX,col='black',type='l',
     lwd=lineweight3,asp=1,ylim=c(0,(1/neweccen)*1.05),
     ylab='',xlab='',yaxt = "n")
points((1+X[sampleX])/2,(MY-maxy[sampleX])/NNX,col='black',type='l',
       lwd=lineweight3)
points(c(min((1+X)/2),min((1+X)/2)),c((MY-miny[1])/NNX,(MY-maxy[1])/NNX),
       col='black',type='l',lwd=lineweight3)
points(c(1,1),c((MY-miny[NNX])/NNX,(MY-maxy[NNX])/NNX),
       col='black',type='l',lwd=lineweight3)
axis(2, at = c(0, 0.2, 0.4, 0.6))
# over plot the egg outline based on XX
points((1+XX)/2,slip2+Carteregg3/2,pch=20,cex=0.1,col=myred)
points((1+XX)/2,slip2-Carteregg3/2,pch=20,cex=0.1,col=myred)
#legend("topleft",legend=paste(as.character(eggdata$eggname[1])),cex=0.6)
#
## ----end of Carter4picture
#
# End of Figure SF12
#

## ----Carterpolarpicture,  fig.align = 'center',   out.width='0.6\\textwidth', fig.height=4.5, fig.pos='tbh', fig.cap = "\\label{Fpolar}  Fit using the method from \\citet{CJ} in red, illustrating the undulating fitted meridian.",   results='hide'----
#
## Start of Figure SF13
#
Carterpolaregg =
function(K1,K2,K3,K4,theta)
{
  as.vector(
    1+
    K1*(cos(2*theta)-1)+
    K2*(-(cos(theta))^3)+
    K3*(-(sin(2*theta))^2)+
    K4*(sin(2*theta)^3)
  )
}

K1=coef(Cay)['t1y']
K2=coef(Cay)['t2y']
K3=coef(Cay)['t3y']
K4=coef(Cay)['t4y']

rgraph=Carterpolaregg(K1,K2,K3,K4,thetapolar)

#par(mai=c(0, 0, 0, 0))

nX=1:length(X)
  sampleX=(nX%%1==0)
plot((1+X[sampleX])/2,(MY-miny[sampleX])/NNX,col='black',type='l',
     lwd=lineweight3,asp=1,ylim=c(0,(1/neweccen)*1.05),
     ylab='',xlab='',yaxt = "n")
points((1+X[sampleX])/2,(MY-maxy[sampleX])/NNX,col='black',type='l',
       lwd=lineweight3)
points(c(min((1+X)/2),min((1+X)/2)),c((MY-miny[1])/NNX,(MY-maxy[1])/NNX),
       col='black',type='l',lwd=lineweight3)
points(c(1,1),c((MY-miny[NNX])/NNX,(MY-maxy[NNX])/NNX),
       col='black',type='l',lwd=lineweight3)
axis(2, at = c(0, 0.2, 0.4, 0.6))

points((1+X2[1:NNX])/2,slip2+(rgraph[1:NNX])*sin(thetapolar)[(NNX+1):(2*NNX)]/2,asp=1,col='red',pch=20,cex=0.1)

points((1+X2[(NNX+1):(2*NNX)])/2,slip2-(rgraph[(NNX+1):(2*NNX)])*sin(thetapolar)[(NNX+1):(2*NNX)]/2,asp=1,col='red',pch=20,cex=0.1)

#points((1+rgraph*cos(thetapolar)+K2)/2,rgraph*sin(thetapolar)/2,asp=1,col='cyan',pch=20,cex=0.1)
#points((1+rgraph*cos(thetapolar)+K2)/2,-rgraph*sin(thetapolar)/2,asp=1,col='cyan',pch=20,cex=0.1)
#
## ----end of Carterpolarpicture
#
# End of Figure SF13
#
dev.off()

